from .copy import Copy
from .delete import Delete
from .echo import Echo
from .move import Move
from .python import Python
from .rename import Rename
from .shell import Shell
from .trash import Trash
